﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Globalization;

namespace calendar
{
    /// <summary>
    /// Displays a calendar on a fixed-width, text-based display.
    /// </summary>
    class Calendar
    {
        const int calWidth = 28; //calendar width (whitespace)

        int currentYear;

        int currentMonth;
        int monthSize;

        Day[] days;
        int today;

        CultureInfo culture = Thread.CurrentThread.CurrentCulture;

        /// <summary>
        /// Creates a calendar using the current system time.
        /// </summary>
        public Calendar()
        {
            initCalendar(DateTime.Now);
        }

        /// <summary>
        /// Creates a calendar using the specified DateTime object
        /// </summary>
        /// <param name="time">Month and Year of the calendar</param>
        public Calendar(DateTime time)
        {
            initCalendar(time);
        }

        public DateTime Date
        {
            get { return new DateTime(currentYear, currentMonth, today); }
            set { initCalendar(value); }
        }

        /// <summary>
        /// Sets up instance variables to be used when writing the calendar to a string.
        /// </summary>
        /// <param name="time">DateTime representing the calendar's month and year</param>
        private void initCalendar(DateTime time)
        {
            currentYear = time.Year;

            currentMonth = time.Month;
            monthSize = DateTime.DaysInMonth(currentYear, currentMonth);

            today = time.Day;

            days = new Day[42]; //42 because we display 6 weeks max (null days are ignored)

            DateTime firstDay = new DateTime(currentYear, currentMonth, 1);
            int start = firstDay.DayOfWeek - culture.DateTimeFormat.FirstDayOfWeek;

            for (int i = start; i < monthSize + start; ++i)
            {
                days[i] = new Day(i - start + 1);
            }

            if (DateTime.Now.Month == currentMonth && DateTime.Now.Year == currentYear)
            {
                days[today + start - 1].Today = true;
            }
        }

        /// <summary>
        /// Writes the calendar to a string that can be printed to a tty
        /// </summary>
        /// <returns>String representing the calendar</returns>
        public override string ToString()
        {
            String calText = "                                  \n";
            String blankLine = " │                            │▐ \n";

            //top of calendar ("graphics")
            calText += "      ▄▄▄▄            ▄▄▄▄       " + '\n';
            calText += " ┌────────────────────────────┐  " + '\n';


            //month
            calText += " │";
            String month = culture.DateTimeFormat.MonthNames[currentMonth - 1] + " " + currentYear;

            //pad the month name to center it
            int paddingSize = (calWidth - month.Length) / 2;
            int unevenPadding = (calWidth - month.Length) % 2;
            month = month.PadLeft(paddingSize + month.Length, ' ');
            month = month.PadRight(paddingSize + unevenPadding + month.Length, ' ');

            calText += month + "│▐ \n";
            calText += blankLine;


            //day labels
            calText += " │";

            //loop through the days of the week and add the first letter of each day
            for (int i = 0; i < culture.DateTimeFormat.DayNames.Length; ++i)
            {
                calText += "  " + culture.DateTimeFormat.DayNames[i].Substring(0, 1) + " ";
            }

            calText += "│▐ \n";


            //separator before we start printing the day numbers
            calText += " ├────────────────────────────┤▐ \n";
            calText += blankLine;


            //print days (most weeks in a month = 6)
            for (int week = 0; week < 6; ++week)
            {
                calText += " │";
                for (int day = week * 7; day < week * 7 + 7; ++day)
                {
                    if (days[day] == null)
                    {
                        calText += "    ";
                    }
                    else
                    {
                        String dayStr = days[day].Number.ToString();
                        if (days[day].Today)
                        {
                            dayStr = "(" + dayStr.PadLeft(2, ' ') + ")";
                        }
                        else
                        {
                            dayStr = dayStr.PadLeft(3, ' ');
                            dayStr += " ";
                        }
                        calText += dayStr;
                    }
                }
                calText += "│▐ \n";
                calText += blankLine;
            }

            calText += " └────────────────────────────┘▐ \n";
            calText += "   ▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀ \n";
            return calText;
        }

        #region Operators
        public static Calendar operator ++(Calendar c)
        {
            return new Calendar(c.Date.AddMonths(1));
        }

        public static Calendar operator --(Calendar c)
        {
            return new Calendar(c.Date.AddMonths(-1));
        }

        public static Calendar operator +(Calendar c, int i)
        {
            return new Calendar(c.Date.AddMonths(i));
        }

        public static Calendar operator -(Calendar c, int i)
        {
            return new Calendar(c.Date.AddMonths(0 - i));
        }
        #endregion
    }
}
